@@ -9,14 +9,13 @@ import uuid
 from collections.abc import Iterator
 from datetime import datetime, timedelta, timezone
 from pathlib import Path, PurePosixPath
-from unittest.mock import patch
+from unittest.mock import Mock, patch
 
 import numpy as np
 import pandas as pd
 import pytest
 import pytz
 from PIL import Image
-from sqlalchemy import Column
 
 import datachain as dc
 from datachain import DataModel, func
@@ -89,8 +88,8 @@ def test_read_storage_non_recursive(cloud_test_catalog):
 
 def test_read_storage_glob(cloud_test_catalog):
     ctc = cloud_test_catalog
-    chain = dc.read_storage(f"{ctc.src_uri}/dogs*", session=ctc.session)
-    assert chain.count() == 4
+    chain = dc.read_storage(f"{ctc.src_uri}/dogs/*", session=ctc.session)
+    assert chain.count() == 3
 
 
 def test_read_storage_as_image(cloud_test_catalog):
@@ -236,22 +235,6 @@ def test_read_storage_dependencies(cloud_test_catalog, cloud_type):
     assert dependencies[0].name == dep_name
 
 
-def test_persist_after_mutate(test_session):
-    chain = (
-        dc.read_values(fib=[1, 1, 2, 3, 5, 8, 13, 21], session=test_session)
-        .map(mod3=lambda fib: fib % 3, output=int)
-        .group_by(
-            cnt=dc.func.count(),
-            partition_by="mod3",
-        )
-        .mutate(x=1)
-        .persist()
-    )
-
-    assert chain.count() == 3
-    assert set(chain.to_values("mod3")) == {0, 1, 2}
-
-
 def test_persist_not_affects_dependencies(tmp_dir, test_session):
     for i in range(4):
         (tmp_dir / f"file{i}.txt").write_text(f"file{i}")
@@ -358,15 +341,24 @@ def test_to_storage(
     file_type,
     num_threads,
 ):
+    mapper = Mock(side_effect=lambda file_path: len(file_path))
+
     ctc = cloud_test_catalog
     df = dc.read_storage(ctc.src_uri, type=file_type, session=test_session)
     if use_map:
-        df.settings(cache=use_cache).map(
-            res=lambda file: file.export(tmp_dir / "output", placement=placement)
-        ).exec()
+        (
+            df.settings(cache=use_cache)
+            .map(mapper, params=["file.path"], output={"path_len": int})
+            .map(res=lambda file: file.export(tmp_dir / "output", placement=placement))
+            .exec()
+        )
     else:
-        df.settings(cache=use_cache).to_storage(
-            tmp_dir / "output", placement=placement, num_threads=num_threads
+        (
+            df.settings(cache=use_cache)
+            .map(mapper, params=["file.path"], output={"path_len": int})
+            .to_storage(
+                tmp_dir / "output", placement=placement, num_threads=num_threads
+            )
         )
 
     expected = {
@@ -387,6 +379,8 @@ def test_to_storage(
         with open(tmp_dir / "output" / file_path) as f:
             assert f.read() == expected[file.name]
 
+    assert mapper.call_count == len(expected)
+
 
 @pytest.mark.parametrize("use_cache", [True, False])
 def test_export_images_files(test_session, tmp_dir, tmp_path, use_cache):
@@ -438,12 +432,12 @@ def test_read_storage_multiple_uris_files(test_session, tmp_dir, tmp_path, use_c
         [
             f"file://{tmp_path}/img1.jpg",
             f"file://{tmp_path}/img2.jpg",
-            f"file://{tmp_dir}/output/*",
+            f"file://{tmp_dir}/output/",
         ]
     )
     assert chain.count() == 4
 
-    chain = dc.read_storage([f"file://{tmp_dir}/output/*"])
+    chain = dc.read_storage([f"file://{tmp_dir}/output/"])
     assert chain.count() == 2
 
 
@@ -473,6 +467,7 @@ def test_read_storage_multiple_uris_cache(cloud_test_catalog):
             session=session,
             update=True,
         ).exec()
+
         assert chain.count() == 11
 
         files = chain.to_values("file")
@@ -765,59 +760,6 @@ def test_read_storage_check_rows(tmp_dir, test_session):
         )
 
 
-def test_mutate_existing_column(test_session):
-    ds = dc.read_values(ids=[1, 2, 3], session=test_session)
-    ds = ds.mutate(ids=Column("ids") + 1)
-
-    assert ds.order_by("ids").to_list() == [(2,), (3,), (4,)]
-
-
-def test_mutate_with_primitives_save_load(test_session):
-    """Test that mutate with primitive values properly persists schema
-    through save/load cycle."""
-    original_data = [1, 2, 3]
-
-    # Create dataset with multiple primitive columns added via mutate
-    ds = dc.read_values(data=original_data, session=test_session).mutate(
-        str_col="test_string",
-        int_col=42,
-        float_col=3.14,
-        bool_col=True,
-    )
-
-    # Verify schema before saving
-    schema = ds.signals_schema.values
-    assert schema.get("str_col") is str
-    assert schema.get("int_col") is int
-    assert schema.get("float_col") is float
-    assert schema.get("bool_col") is bool
-
-    ds.save("test_mutate_primitives")
-
-    # Load the dataset back
-    loaded_ds = dc.read_dataset("test_mutate_primitives", session=test_session)
-
-    # Verify schema after loading
-    loaded_schema = loaded_ds.signals_schema.values
-    assert loaded_schema.get("str_col") is str
-    assert loaded_schema.get("int_col") is int
-    assert loaded_schema.get("float_col") is float
-    assert loaded_schema.get("bool_col") is bool
-
-    # Verify data integrity
-    results = set(loaded_ds.to_list())
-    assert len(results) == 3
-
-    # Expected tuples: (data, str_col, int_col, float_col, bool_col)
-    expected_results = {
-        (1, "test_string", 42, 3.14, True),
-        (2, "test_string", 42, 3.14, True),
-        (3, "test_string", 42, 3.14, True),
-    }
-
-    assert results == expected_results
-
-
 @pytest.mark.parametrize("processes", [False, 2, True])
 @pytest.mark.xdist_group(name="tmpfile")
 def test_parallel(processes, test_session_tmpfile):
@@ -880,7 +822,7 @@ def test_udf_parallel(cloud_test_catalog_tmpfile):
 
     chain = (
         dc.read_storage(cloud_test_catalog_tmpfile.src_uri, session=session)
-        .settings(parallel=-1)
+        .settings(parallel=True)
         .map(name_len, params=["file.path"], output={"name_len": int})
         .select("file.path", "name_len")
     )
@@ -1054,7 +996,7 @@ def test_udf_parallel_exec_error(cloud_test_catalog_tmpfile):
         dc.read_storage(cloud_test_catalog_tmpfile.src_uri, session=session)
         .filter(dc.C("file.size") < 13)
         .filter(dc.C("file.path").glob("cats*") | (dc.C("file.size") < 4))
-        .settings(parallel=-1)
+        .settings(parallel=True)
         .map(name_len_error, params=["file.path"], output={"name_len": int})
     )
 
@@ -1158,7 +1100,7 @@ def test_udf_parallel_interrupt(cloud_test_catalog_tmpfile, capfd):
         dc.read_storage(cloud_test_catalog_tmpfile.src_uri, session=session)
         .filter(dc.C("file.size") < 13)
         .filter(dc.C("file.path").glob("cats*") | (dc.C("file.size") < 4))
-        .settings(parallel=-1)
+        .settings(parallel=True)
         .map(name_len_interrupt, params=["file.path"], output={"name_len": int})
     )
     if os.environ.get("DATACHAIN_DISTRIBUTED"):
@@ -1441,7 +1383,7 @@ def test_gen_parallel(cloud_test_catalog_tmpfile):
 
     chain = (
         dc.read_storage(cloud_test_catalog_tmpfile.src_uri, session=session)
-        .settings(parallel=-1)
+        .settings(parallel=True)
         .gen(gen=func, params=["file"], output={"val": str})
         .order_by("val")
     )
@@ -2409,3 +2351,22 @@ def test_agg_sample(catalog_tmpfile, parallel, sample):
     records = list(ds.to_records())
     assert len(records) == sample
     assert all(row["count"] == 1 for row in records)
+
+
+def test_batch_for_map(test_session):
+    # Create a chain with batch settings
+    chain = dc.read_values(x=list(range(100)), session=test_session)
+    chain_with_settings = chain.settings(batch_size=15)
+
+    def add_one(x):
+        return x + 1
+
+    result = chain_with_settings.map(add_one, output={"result": int})
+
+    results = [
+        r[0] for r in result.to_iter("result")
+    ]  # Access the first element of each tuple
+
+    assert len(results) == 100
+
+    assert set(results) == set(range(1, 101))
