@@ -11,6 +11,7 @@ from datachain.func import (
     int_hash_64,
     isnone,
     literal,
+    not_,
     or_,
 )
 from datachain.func.array import contains
@@ -368,6 +369,18 @@ def test_and_func_mutate(chain):
     ]
 
 
+@skip_if_not_sqlite
+def test_not_func_mutate(chain):
+    res = chain.mutate(test=ifelse(not_(dc.C("num") > 3), "Match", "Not Match"))
+    assert list(res.order_by("num").to_values("test")) == [
+        "Match",
+        "Match",
+        "Match",
+        "Not Match",
+        "Not Match",
+    ]
+
+
 def test_xor():
     rnd1, rnd2 = rand(), rand()
 
